locals {
  # Configure policy identifiers for aws_iam_policy_document argument
  lambda_identifiers        = var.lambda_role ? "lambda.amazonaws.com" : ""
  firehose_identifiers      = var.firehose_role ? "firehose.amazonaws.com" : ""
  stepfunctions_identifiers = var.stepfunctions_role ? "states.amazonaws.com" : ""
  glue_identifiers          = var.glue_role ? "glue.amazonaws.com" : ""
  dax_identifiers           = var.dax_role ? "dax.amazonaws.com" : ""
  service_identifiers       = compact([local.firehose_identifiers, local.lambda_identifiers, local.glue_identifiers, local.stepfunctions_identifiers, local.dax_identifiers])
  is_service_role           = length(local.service_identifiers) == 0 ? false : true

  # Configure policy ARN for aws_iam_role argument
  lambda_policy_arn        = var.lambda_role ? aws_iam_policy.lambda-default-policy[0].arn : ""
  firehose_policy_arn      = var.firehose_role ? aws_iam_policy.firehose-default-policy[0].arn : ""
  stepfunctions_policy_arn = var.stepfunctions_role ? aws_iam_policy.stepfunctions-default-policy[0].arn : ""
  dax_policy_arn           = var.dax_role ? aws_iam_policy.dax-default-policy[0].arn : ""
  service_policy_arns      = compact([local.firehose_policy_arn, local.lambda_policy_arn, local.stepfunctions_policy_arn, var.s3_policy_arn, var.kinesis_policy_arn, local.dax_policy_arn])
}

data "aws_region" "current" {}

# AWS IAM Role
resource "aws_iam_role" "role" {
  name = var.role_name

  force_detach_policies = true

  assume_role_policy  = local.is_service_role ? data.aws_iam_policy_document.service-trust-relationship[0].json : data.aws_iam_policy_document.k8s-trust-relationship[0].json
  managed_policy_arns = local.is_service_role ? concat(var.managed_policies, var.policies, local.service_policy_arns) : concat(var.managed_policies, var.policies)

  tags = var.tags

  lifecycle {
    ignore_changes = [permissions_boundary]
  }
}

#############################################################
##                     Trust Documents                     ##
#############################################################

# This trust relationship will be added if k8s app role is getting created
data "aws_iam_policy_document" "k8s-trust-relationship" {
  count = local.is_service_role ? 0 : 1
  statement {
    actions = ["sts:AssumeRole"]

    principals {
      type        = "Service"
      identifiers = var.rds_role ? ["ec2.amazonaws.com", "rds.amazonaws.com", "export.rds.amazonaws.com"] : ["ec2.amazonaws.com"]
    }

    principals {
      type        = "AWS"
      identifiers = var.trust_relationship_arns
    }

    principals {
      type        = "AWS"
      identifiers = ["${var.aws_arn_prefix}:iam::${var.aws_account_number}:root"]
    }
  }
}

# This trust relationship will be added only if a service role is being created
data "aws_iam_policy_document" "service-trust-relationship" {
  count = local.is_service_role ? 1 : 0

  statement {
    actions = ["sts:AssumeRole"]

    principals {
      type        = "Service"
      identifiers = local.service_identifiers
    }
  }
}

#############################################################
##                    Lambda IAM Policy                    ##
#############################################################

resource "aws_iam_policy" "lambda-default-policy" {
  count       = var.lambda_role ? 1 : 0
  name        = "${var.role_name}-default-policy"
  path        = "/"
  description = "Lambda Role Default Policy for ${var.role_name}"
  policy = jsonencode({
    "Version" : "2012-10-17",
    "Statement" : [
      {
        "Effect" : "Allow",
        "Action" : [
          "logs:CreateLogGroup",
          "logs:CreateLogStream",
          "logs:PutLogEvents",
          "ec2:CreateNetworkInterface",
          "ec2:DescribeNetworkInterfaces",
          "ec2:DeleteNetworkInterface",
          "ec2:AssignPrivateIpAddresses",
          "ec2:UnassignPrivateIpAddresses"
        ],
        "Resource" : "*"
      }
    ]
  })

  tags = var.tags

  lifecycle {
    ignore_changes = [
      description,
    ]
  }
}

#############################################################
##                   Firehose IAM Policy                   ##
#############################################################

resource "aws_iam_policy" "firehose-default-policy" {
  count       = var.firehose_role ? 1 : 0
  name        = "${var.role_name}-default-policy"
  path        = "/"
  description = "Firehose Role Default Policy for ${var.role_name}"
  policy      = data.aws_iam_policy_document.firehose[0].json

  tags = var.tags

  lifecycle {
    ignore_changes = [
      description,
    ]
  }
}

data "aws_iam_policy_document" "firehose" {
  count = var.firehose_role ? 1 : 0

  # Perform actions on all Firehose streams
  statement {
    sid       = "manageFirehoseStreams"
    effect    = "Allow"
    actions   = ["firehose:*"]
    resources = ["*"]
  }

  statement {
    sid       = "listKinesisStreams"
    effect    = "Allow"
    actions   = ["kinesis:ListStreams"]
    resources = ["*"]
  }

  # List/Get Cloudwatch metrics
  statement {
    sid    = "listMetrics"
    effect = "Allow"
    actions = [
      "cloudwatch:GetMetricStatistics",
      "cloudwatch:ListMetrics"
    ]
    resources = ["*"]
  }

  # Get Logs
  statement {
    sid       = "getLogEvents"
    effect    = "Allow"
    actions   = ["logs:GetLogEvents"]
    resources = ["*"]
  }
}

####################################################################
##                    StepFunctions IAM Policy                    ##
####################################################################

resource "aws_iam_policy" "stepfunctions-default-policy" {
  count       = var.stepfunctions_role ? 1 : 0
  name        = "${var.role_name}-stepfunctions-default-policy"
  path        = "/"
  description = "Step Functions Role Default Policy for ${var.role_name}"
  policy = jsonencode({
    "Version" : "2012-10-17",
    "Statement" : [
      {
        "Effect" : "Allow",
        "Action" : [
          "logs:CreateLogDelivery",
          "logs:GetLogDelivery",
          "logs:UpdateLogDelivery",
          "logs:DeleteLogDelivery",
          "logs:ListLogDeliveries",
          "logs:PutResourcePolicy",
          "logs:DescribeResourcePolicies",
          "logs:DescribeLogGroups",
          "lambda:InvokeFunction"
        ],
        "Resource" : "*"
      }
    ]
  })

  tags = var.tags

  lifecycle {
    ignore_changes = [
      description,
    ]
  }
}

##########################################################
##                Assume Role Policy                    ##
##########################################################

resource "aws_iam_role_policy" "assume-default-policy" {
  count = var.assume_role ? 1 : 0
  name  = "${var.role_name}-assume-default-policy"
  role  = aws_iam_role.role.id

  policy = jsonencode({
    "Version" : "2012-10-17",
    "Statement" : [
      {
        "Effect" : "Allow",
        "Action" : "sts:AssumeRole",
        "Resource" : "*"
    }]
  })
}

##########################################################
##                    RDS IAM Policy                    ##
##########################################################

resource "aws_iam_role_policy" "rds-default-policy" {
  count = var.rds_role ? 1 : 0
  name  = "${var.role_name}-rds-default-policy"
  role  = aws_iam_role.role.id

  policy = jsonencode({
    "Version" : "2012-10-17",
    "Statement" : [
      {
        "Effect" : "Allow",
        "Action" : [
          "iam:PassRole",
          "iam:GetRole"
        ],
        "Resource" : aws_iam_role.role.arn
      }
    ]
  })
}

####################################################################
##                    DAX IAM Policy                              ##
####################################################################

data "aws_iam_policy_document" "dax-default-policy-doc" {
  statement {
    sid    = "EnableDynamoDBAccess"
    effect = "Allow"
    actions = [
      "dynamodb:DescribeTable",
      "dynamodb:PutItem",
      "dynamodb:GetItem",
      "dynamodb:UpdateItem",
      "dynamodb:DeleteItem",
      "dynamodb:Query",
      "dynamodb:Scan",
      "dynamodb:BatchGetItem",
      "dynamodb:BatchWriteItem",
      "dynamodb:ConditionCheckItem"
    ]

    resources = [
      for dynamodb_table_arn in var.dynamodb_tables_arns :
      dynamodb_table_arn
    ]
  }
}

resource "aws_iam_policy" "dax-default-policy" {
  count       = var.dax_role ? 1 : 0
  name        = "${var.role_name}-dax-default-policy"
  path        = "/"
  description = "DAX Role Default Policy for ${var.role_name}"
  policy      = data.aws_iam_policy_document.dax-default-policy-doc.json

  tags = var.tags

  lifecycle {
    ignore_changes = [
      description,
    ]
  }
}

#############################################################
##             Dataforwarder IAM Policy                    ##
#############################################################

resource "aws_iam_role_policy" "dataforwarder-permission-sts-policy" {
  count = var.dataforwarder_role ? 1 : 0

  name = "${var.role_name}-permission-sts-policy"
  role = aws_iam_role.role.id
  policy = jsonencode({
    "Version" : "2012-10-17",
    "Statement" : [
      {
        "Effect" : "Allow",
        "Action" : [
          "sts:AssumeRole"
        ],
        "Resource" : data.aws_iam_role.dataforwarder_static_s3_role[0].arn
      }
    ]
  })
}

data "aws_iam_role" "dataforwarder_static_s3_role" {
  count = var.dataforwarder_role ? 1 : 0
  name  = "mcs2-psc-data-forwarder-s3"
}
